//+------------------------------------------------------------------+
//|                                                  SOM_ex1-hex.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

// input parameter, used to show hexagonal cells
input bool HexagonalCell=true;
// input parameter, used to show borders
input bool ShowBorders=true;

//--- include cIntBMP library
#include <cIntBMP.mqh>
//+------------------------------------------------------------------+
//| CSOMNode class                                                   |
//+------------------------------------------------------------------+
class CSOMNode
  {
protected:
   int               m_x1;
   int               m_y1;
   int               m_x2;
   int               m_y2;

   double            m_x;
   double            m_y;
   double            m_weights[];
public:
   //--- class constructor
                     CSOMNode();
   //--- class destructor
                    ~CSOMNode();
   //--- node initialization
   void              InitNode(int x1,int y1,int x2,int y2);
   //--- return coordinates of the node's center
   double            X()  const { return(m_x);}
   double            Y()  const { return(m_y);}
   //--- returns the node coordinates
   void              GetCoordinates(int &x1,int &y1,int &x2,int &y2);
   //--- returns the value of weight_index component of weight's vector
   double            GetWeight(int weight_index);
   //--- returns the squared distance between the node weights and specified vector
   double            CalculateDistance(double &vector[]);
   //--- adjust weights of the node
   void              AdjustWeights(double &vector[],double learning_rate,double influence);
  };
//+------------------------------------------------------------------+
//| Class constructor                                                |
//+------------------------------------------------------------------+
void CSOMNode::CSOMNode()
  {
   m_x1=0;
   m_y1=0;
   m_x2=0;
   m_y2=0;
   m_x=0.0;
   m_y=0.0;
  };
//+------------------------------------------------------------------+
//| Node initialization method                                       |
//+------------------------------------------------------------------+
void CSOMNode::InitNode(int x1,int y1,int x2,int y2)
  {
//--- set node coordinates
   m_x1=x1;
   m_y1=y1;
   m_x2=x2;
   m_y2=y2;
//--- calculation of node's center
   m_x=x1+MathAbs((x2-x1))/2;
   m_y=y1+MathAbs((y2-y1))/2;
//--- prepare weights array
   ArrayResize(m_weights,3);
//--- initialize weights with random values
   for(int i=0; i<3; i++) {m_weights[i]=255.0*rand()/32768;}
  };
//+------------------------------------------------------------------+
//| Class destructor                                                 |
//+------------------------------------------------------------------+
void CSOMNode::~CSOMNode()
  {
   ArrayResize(m_weights,0);
  };
//+------------------------------------------------------------------+
//| Returns the value of weight_index component of weight's vector   |
//+------------------------------------------------------------------+
double CSOMNode::GetWeight(int weight_index)
  {
   if(weight_index>=0 && weight_index<3)
     {
      return(m_weights[weight_index]);
     }
   else return(0);
  }
//+------------------------------------------------------------------+
//| Returns coordinates of the node                                  |
//+------------------------------------------------------------------+
void CSOMNode::GetCoordinates(int &x1,int &y1,int &x2,int &y2)
  {
   x1=m_x1;
   y1=m_y1;
   x2=m_x2;
   y2=m_y2;
  }
//+------------------------------------------------------------------+
//| Returns the value of squared distance                            |
//| between the node weights and specified vector                    |
//+------------------------------------------------------------------+
double CSOMNode::CalculateDistance(double &vector[])
  {
   double dist_sqr=0;
   for(int i=0; i<3; i++)
     {
      dist_sqr=dist_sqr+(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
     }
   return(dist_sqr);
  };
//+------------------------------------------------------------------+
//| Adjust weights of the node                                       |
//+------------------------------------------------------------------+
void CSOMNode::AdjustWeights(double &vector[],double learning_rate,double influence)
  {
   for(int i=0; i<3; i++)
     {
      m_weights[i]=m_weights[i]+learning_rate*influence*(vector[i]-m_weights[i]);
     }
  }
//+------------------------------------------------------------------+
//| CSOM class                                                       |
//+------------------------------------------------------------------+
class CSOM
  {
protected:
   //--- class for using of bmp images
   cIntBMP           m_bmp;
   //--- grid mode
   int               m_gridmode;
   //--- bmp image size
   int               m_xsize;
   int               m_ysize;
   //--- number of nodes
   int               m_xcells;
   int               m_ycells;
   //--- array with nodes
   CSOMNode          m_som_nodes[];
   //--- total items in training set
   int               m_total_training_sets;
   //--- training set array
   double            m_training_sets_array[];
protected:
   //--- radius of the neighbourhood (used for training)
   double            m_map_radius;
   //--- time constant (used for training)
   double            m_time_constant;
   //--- initial learning rate (used for training)
   double            m_initial_learning_rate;
   //--- iterations (used for training)
   int               m_iterations;

public:
   //--- class constructor
                     CSOM();
   //--- class destructor
                    ~CSOM();
   //--- net initialization
   void              InitParameters(int iterations,int xcells,int ycells,int bmpwidth,int bmpheight);
   //--- finds the best matching node, closest to the specified vector
   int               BestMatchingNode(double &vector[]);
   //--- train method
   void              Train();
   //--- render method
   void              Render();
   //--- shows the bmp image on the chart
   void              ShowBMP(bool back);
   //--- adds a vector to training set
   void              AddVectorToTrainingSet(double &vector[]);
   //--- shows the pattern title
   void              ShowPattern(double c1,double c2,double c3,string name);
   //--- adds a pattern to training set
   void              AddTrainPattern(double c1,double c2,double c3);
   //--- returns the RGB components of the color
   void              ColToRGB(int col,int &r,int &g,int &b);
   //--- returns the color by RGB components
   int               RGB256(int r,int g,int b) const {return(r+256*g+65536*b);      }
   //--- deletes image from the chart
   void              NetDeinit();
  };
//+------------------------------------------------------------------+
//| Class constructor                                                |
//+------------------------------------------------------------------+
void CSOM::CSOM()
  {
   m_iterations=10;
   m_initial_learning_rate=0.1;
   m_total_training_sets=0;
   ArrayResize(m_training_sets_array,0);
  }
//+------------------------------------------------------------------+
//| Class destructor                                                 |
//+------------------------------------------------------------------+
void CSOM::~CSOM()
  {
   ArrayResize(m_training_sets_array,0);
  }
//+------------------------------------------------------------------+
//| Kohonen map initialization                                       |
//+------------------------------------------------------------------+
void CSOM::InitParameters(int iterations,int xcells,int ycells,int bmpwidth,int bmpheight)
  {
   m_xsize=bmpwidth;
   m_ysize=bmpheight;

   m_xcells=xcells;
   m_ycells=ycells;

//--- save current grid mode
   m_gridmode=int(ChartGetInteger(0,CHART_SHOW_GRID,0));

//--- set number of iterations
   m_iterations=iterations;
   int cellwidth=m_xsize/m_xcells;
   int cellheight=m_ysize/m_ycells;

//--- create BMP image
   if(!HexagonalCell) {m_bmp.Create(m_xsize,m_ysize,clrWhite);}
   else m_bmp.Create(m_xsize+cellwidth,m_ysize+cellheight/2,clrWhite);

//--- prepare nodes array
   ArrayResize(m_som_nodes,xcells*ycells);
   int ind=0;
//--- initialize nodes
   for(int i=0; i<m_xcells; i++)
     {
      for(int j=0; j<m_ycells; j++)
        {
         m_som_nodes[ind].InitNode(i*cellwidth,j*cellheight,(i+1)*cellwidth,(j+1)*cellheight);
         ind++;
        }
     }
//--- calculate initial neighbourhood radius
   if(m_xsize>m_ysize) {m_map_radius=m_xsize/2.0;}
   else {m_map_radius=m_ysize/2.0;}
//Print("m_map_radius=",m_map_radius);
   m_time_constant=1.0*m_iterations/MathLog(m_map_radius);
   Print("m_time_constant=",m_time_constant);
  }
//+------------------------------------------------------------------+
//| Finds index of the best mathching node                           |
//+------------------------------------------------------------------+
int CSOM::BestMatchingNode(double &vector[])
  {
   int    min_ind=0;
   double min_dist=m_som_nodes[min_ind].CalculateDistance(vector);
   int total_nodes= ArraySize(m_som_nodes);
   for(int i=0; i<total_nodes; i++)
     {
      if(m_som_nodes[i].CalculateDistance(vector)<min_dist)
        {
         min_dist=m_som_nodes[i].CalculateDistance(vector);
         min_ind=i;
        };
     }
   return(min_ind);
  }
//+------------------------------------------------------------------+
//| Train net                                                        |
//+------------------------------------------------------------------+
void CSOM::Train()
  {
   double learning_rate=m_initial_learning_rate;
   int iter=0;
   double datavector[];
   ArrayResize(datavector,3);

   int total_nodes=ArraySize(m_som_nodes);

   int frame=0;
   do
     {
      //--- random selection of the vector from training set
      int ind=int(1.0*m_total_training_sets*rand()/32768);
      //--- get its components
      datavector[0]=m_training_sets_array[3*(ind)];
      datavector[1]=m_training_sets_array[3*(ind)+1];
      datavector[2]=m_training_sets_array[3*(ind)+2];
      //--- find the index of winning node, closest to datavector
      int winningnode=BestMatchingNode(datavector);
      //--- recalculate current neighbourhood radius
      double neighbourhood_radius=m_map_radius*MathExp(-1.0*iter/m_time_constant);

      //--- proceed all nodes
      for(int i=0; i<total_nodes; i++)
        {
         //--- calc squared distance to the node with winningnode index
         double DistToNodeSqr=(m_som_nodes[winningnode].X()-m_som_nodes[i].X())*(m_som_nodes[winningnode].X()-m_som_nodes[i].X())
                             +(m_som_nodes[winningnode].Y()-m_som_nodes[i].Y())*(m_som_nodes[winningnode].Y()-m_som_nodes[i].Y());
         //--- squared neighbourhood radius
         double WS=neighbourhood_radius*neighbourhood_radius;
         //--- if the node is inside the neighbourhood radius,recalc
         if(DistToNodeSqr<WS)
           {
            //--- calculate squared neighbourhood radius
            double influence=MathExp(-DistToNodeSqr/(2*WS));
            //--- if the node inside the neighbourhood, adjust its weights
            m_som_nodes[i].AdjustWeights(datavector,learning_rate,influence);
           }
        }
      //--- calc influence
      learning_rate=m_initial_learning_rate*MathExp(-1.0*iter/m_iterations);
      //--- increase iterations counter
      iter++;
      //--- show dynamics of Kohonen map training
      //--- render and show image after each 100 iterations
      if(iter%100==0)
        {
         frame++;
         Print("Iteration ",iter," of ",m_iterations," ",int(iter*100.0/m_iterations),"%");
         Render();
         ShowBMP(false);
        }
     }
   //--- repeat until iter<m_iterations
   while(iter<m_iterations);

   Print("Train complete.");
  }
//+------------------------------------------------------------------+
//| Shows bmp image on the chart                                     |
//+------------------------------------------------------------------+
void CSOM::ShowBMP(bool back)
  {
//--- disable show grid  
   ChartSetInteger(0,CHART_SHOW_GRID,0);
//--- save bmp image to file
   m_bmp.Save("bmpimg",true);
//--- show bmp image on the chart 
   m_bmp.Show(0,0,"bmpimg","SOM");
//--- set background property
   ObjectSetInteger(0,"SOM",OBJPROP_BACK,back);
  }
//+------------------------------------------------------------------+
//| Adds a vector to training set                                    |
//+------------------------------------------------------------------+
void CSOM::AddVectorToTrainingSet(double &vector[])
  {
   m_total_training_sets++;
   ArrayResize(m_training_sets_array,3*m_total_training_sets);
   for(int i=0; i<3; i++)
     {
      m_training_sets_array[3*(m_total_training_sets-1)+i]=vector[i];
     }
  }
//+------------------------------------------------------------------+
//| Shows the title of the specified pattern                         |
//+------------------------------------------------------------------+
void CSOM::ShowPattern(double c1,double c2,double c3,string name)
  {
   double datavector[3];
   datavector[0]=c1;
   datavector[1]=c2;
   datavector[2]=c3;
   int WinningNodeIndex=BestMatchingNode(datavector);
   int ind=BestMatchingNode(datavector);

   int x1,y1,x2,y2;
   m_som_nodes[ind].GetCoordinates(x1,y1,x2,y2);

   int r= int(m_som_nodes[ind].GetWeight(0));
   int g= int(m_som_nodes[ind].GetWeight(1));
   int b= int(m_som_nodes[ind].GetWeight(2));

   if(HexagonalCell)
     {
      //--- hexagonal cells
      int x_size=int (MathAbs(x2-x1));
      int y_size=int (MathAbs(y2-y1));
      y1=y1+y_size/4;
      y2=y2+y_size/4;
     }

   int xc=(x1+x2)/2;
   int yc=(y1+y2)/2;

   if(r*r+g*g+b*b<=128*128)
     {
      m_bmp.TypeText(xc+1,yc+1,name,clrBlack);
      m_bmp.TypeText(xc,yc,name,clrWhite);
     }
   else
     {
      m_bmp.TypeText(xc+1,yc+1,name,clrBlack);
      m_bmp.TypeText(xc,yc,name,clrWhite);
     }
   Print("Pattern ",name,": has best matching node index =",ind);
  }
//+------------------------------------------------------------------+
//| Adds a pattern to training set                                   |
//+------------------------------------------------------------------+
void CSOM::AddTrainPattern(double c1,double c2,double c3)
  {
   double vector[3];
   vector[0]=c1;
   vector[1]=c2;
   vector[2]=c3;
   AddVectorToTrainingSet(vector);
  }
//+------------------------------------------------------------------+
//| Returns the RGB components of the color                          |
//+------------------------------------------------------------------+
void CSOM::ColToRGB(int col,int &r,int &g,int &b)
  {
   r=col & 0xFF;
   g=(col>>8)& 0xFF;
   b=(col>>16)& 0xFF;
  }
//+------------------------------------------------------------------+
//| Render method (to m_bmp image)                                   |
//+------------------------------------------------------------------+
void CSOM::Render()
  {
   int ind=0;
   for(int i=0; i<m_xcells; i++)
     {
      for(int j=0; j<m_ycells; j++)
        {
         int x1,y1,x2,y2;
         m_som_nodes[ind].GetCoordinates(x1,y1,x2,y2);
         int r = int(m_som_nodes[ind].GetWeight(0));
         int g = int(m_som_nodes[ind].GetWeight(1));
         int b = int(m_som_nodes[ind].GetWeight(2));
         int col=RGB256(r,g,b);

         if(!HexagonalCell)
           {
            //--- cells as rectangles
            m_bmp.DrawRectangle(x1,y1,x2,y2,col,true);
            if(ShowBorders) m_bmp.DrawRectangle(x1,y1,x2,y2,clrBlack,false);
           }
         else
           {
            //--- hexagonal cells
            int x_size=int (MathAbs(x2-x1));
            int y_size=int (MathAbs(y2-y1));
            if(j%2==0)
              {
               x1=x1+x_size/2;
               x2=x2+x_size/2;
              }
            y1=y1+y_size/4;
            y2=y2+y_size/4;

            m_bmp.DrawRectangle(x1,y1+y_size/4,x2,y2-y_size/4,col,true);
            m_bmp.DrawTriangle(x1,y1+y_size/4,x1+x_size/2,y1-y_size/4,x2,y1+y_size/4,col,true);
            m_bmp.DrawTriangle(x1,y2-y_size/4,x2,y2-y_size/4,x1+x_size/2,y2+y_size/4,col,true);

            if(ShowBorders)
              {
               //--- show borders of hexagonal cells
               int bcol=clrBlack;
               m_bmp.DrawLine(x1,y1+y_size/4,x1+x_size/2,y1-y_size/4,bcol);
               m_bmp.DrawLine(x1+x_size/2,y1-y_size/4,x2,y1+y_size/4,bcol);

               m_bmp.DrawLine(x1,y2-y_size/4,x1+x_size/2,y2+y_size/4,bcol);
               m_bmp.DrawLine(x2,y2-y_size/4,x2-x_size/2,y2+y_size/4,bcol);

               m_bmp.DrawLine(x1,y1+y_size/4,x1,y2-y_size/4,bcol);
               m_bmp.DrawLine(x2,y1+y_size/4,x2,y2-y_size/4,bcol);
              }

           }
         ind++;
        }
     }
   return;
  }
//+------------------------------------------------------------------+
//| Net deinitialization                                             |
//+------------------------------------------------------------------+
void CSOM::NetDeinit()
  {
//--- restore old CHART_SHOW_GRID mode
   ChartSetInteger(0,CHART_SHOW_GRID,m_gridmode);
//--- delete SOM object from the chart
   ObjectDelete(0,"SOM");
  }

CSOM KohonenMap;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
void OnInit()
  {
  MathSrand(200);
//--- initialize net, 10000 iterations will be used for training
//--- the net contains 15x20 nodes, bmp image size 400x400
   KohonenMap.InitParameters(10000,15,20,400,400);
//-- add RGB-components of each color from the training set
   KohonenMap.AddTrainPattern(255,  0,  0);   // Red
   KohonenMap.AddTrainPattern(  0,128,  0);   // Green
   KohonenMap.AddTrainPattern(  0,  0,255);   // Blue
   KohonenMap.AddTrainPattern(  0,100,  0);   // Dark green
   KohonenMap.AddTrainPattern(  0,  0,139);   // Dark blue
   KohonenMap.AddTrainPattern(255,255,  0);   // Yellow
   KohonenMap.AddTrainPattern(255,165,  0);   // Orange
   KohonenMap.AddTrainPattern(128,  0,128);   // Purple
//--- train net
   KohonenMap.Train();
//--- render map to bmp
   KohonenMap.Render();
//--- show patterns and titles for each color
   KohonenMap.ShowPattern(255,  0,  0,"Red");
   KohonenMap.ShowPattern(  0,128,  0,"Green");
   KohonenMap.ShowPattern(  0,  0,255,"Blue");
   KohonenMap.ShowPattern(  0,100,  0,"Dark green");
   KohonenMap.ShowPattern(  0,  0,139,"Dark blue");
   KohonenMap.ShowPattern(255,255,  0,"Yellow");
   KohonenMap.ShowPattern(255,165,  0,"Orange");
   KohonenMap.ShowPattern(128,  0,128,"Purple");
//--- show bmp image on the chart
   KohonenMap.ShowBMP(false);
//---
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   KohonenMap.NetDeinit();
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
  }
//+------------------------------------------------------------------+
